/*
 * a NewYacc (c) grammar for parsing WISE configuration
 * files (J. Callahan 1995-09-18)
 *
 */

%{

extern char* action();
extern char* command();
extern char* msql_binary();
extern char* msql_current();
extern char* create_passwd();

char *fname,*host;

%}

%tag WISE PROJECT ALLPROJ PRJTDL ROLE VIEWACCESS COMP HEADERS
%tag ACCESS WHEN INIT NEWACCESS TDLACTION ACTION VIEWACTION QUERY TDLVIEW LIST ROOT LBL
%tag DATABASE DBDECL DBLOG DBDATA SCRIPT NAMES DUMP USERS MSQLA
%token DATABASE HOST PORT ORGANIZATION IMAGE ROOT BIN PROJECT HTTPROOT
%token TEXT TEXTAREA DATE SELECT RADIO CHECKBOX HTML LOG INT PASSWD ACCESS
%token ROLE TDLVIEW VIEWACCESS ISA ON WHEN SEES ALL FIELD IS HIDDEN TO
%token HAS HAVE NEWACCESS TDLACTION VIEWACTION CAN CALL TDL SUBMITISSUE
%token READONLY READWRITE WRITEONLY STRING ID NUM NEWISSUE DELETEISSUE
%token OPERATOR AND OR LIKE LABEL BACKGROUND METRICS

%start file

%%

file		:	spec
			[ (MSQLA) !host #1 ]
			[ (WISE)
			  &1(HEADERS)
			  "#define WISELOGS\t\"" "wise" "/logs/\"\n"
			  &1(ROOT) "\n"
			  "#define WISE_TDL\t\"" "wise" "-bin/tdl\"\n"
			  "#define WISE_VIEW\t\"" "wise" "-bin/view\"\n"
			  "#define WISE_UPDATE\t\"" "wise" "-bin/update\"\n"
			  "#define WISE_NEW\t\"" "wise" "-bin/new\"\n"
			  "#define WISE_DELETE\t\"" "wise" "-bin/delete\"\n\n"
			  "#define WISE_VIEW_UPDATE\t\"update\"\n"
			  "#define WISE_VIEW_NEW\t\t\"new\"\n"
			  "#define WISE_VIEW_DELETE\t\"delete\"\n\n"
			  "#include \"wisedefs.h\"\n\n"
			  "#endif WISE\n\n"
			  "#ifdef WISEMAIN\n\n"
			  &1(PROJECT)
   			  &1(PRJTDL)
			  "WiseProject wiseproject[] = {\n"
			  &1(ALLPROJ)
			  "  { NULL, NULL, NULL, NULL, NULL, NULL, NULL }\n"
			  "};\n\n"
                          "WiseUserRole  wiseuserroles[] = {\n"
			  &1(ROLE)
                          "  { NULL, NULL, NULL }\n" 
                 	  "};\n\n"
			 "WiseTdlAccess  wisetdlaccess[] = {\n"
 			  &1(TDLVIEW)
                          "  { NULL, NULL, NULL }\n" 
                 	  "};\n\n"
			 "WiseViewAccess  wiseviewaccess[] = {\n"
 			  &1(VIEWACCESS)
                          "  { NULL, NULL, NULL, NULL, NULL, NULL }\n" 
                 	  "};\n\n"
			 "WiseViewAccess  wisenewaccess[] = {\n"
 			  &1(NEWACCESS)
                          "  { NULL, NULL, NULL, NULL, NULL, NULL }\n" 
                 	  "};\n\n" 
 			 "WiseAction wisetdlactions[] = {\n"
 			  &1(TDLACTION)
                          "  { NULL, NULL, NULL, NULL, NULL }\n" 
                 	  "};\n\n" 
 			 "WiseAction wiseviewactions[] = {\n"
 			  &1(VIEWACTION)
                          "  { NULL, NULL, NULL, NULL, NULL }\n" 
                 	  "};\n\n"  
			"#else\n" 
			]
			[ (DATABASE)
			  &1(DBDROP) "\n"
			  &1(DBDATA) "\n"
			  &1(DBLOG) "\n"
			  &1(DBDECL)
			]
		;

spec		:	/* empty */
		|	unit spec
		;

unit		:	HOST id
			[ (HEADERS) "#define WISEHOST\t\"" #2() "\"\n" ]
			[ (MSQLA) @host=#2() ]
		|	PORT num
			[ (HEADERS) "#define WISEPORT\t\"" #2() "\"\n" ]
		|	ORGANIZATION qid
			[ (HEADERS) "#define WISEORGNAME\t\"" #2() "\"\n" ]
		|	IMAGE id
			[ (HEADERS) "#define WISEORGIMAGE\t\"" #2() "\"\n" ]
		|	ROOT id
			[ (ROOT)  "#define WISEPATHLOG\t\"" #2() "/logs/\"\n" 
				  "#define WISE_METRICDIR\t\"" #2() "/metrics\"\n" ]
		|	BACKGROUND id
			[ (HEADERS) "#define WISEBKGND\t\"" #2() "\"\n" ]
		|	METRICS id
			[ (HEADERS) "#define WISEMETRICS\t\"" #2() "\"\n" ]
		|	project
		|	tdl
		|	role 
 		|	tdlview
		|	viewaccess
		|	newaccess
		|	tdlaction
		|	viewaction
		;

project		:	PROJECT id STRING '{' DATABASE id HOST id PORT num IMAGE id fields '}'
			[ (PROJECT)
			  "WiseField wise_" #2() "project[] = {\n"
			  #13
			  "  { NULL, NULL, NULL }\n"
			  "};\n\n"
			]
			[ (MSQLA)
			  "database=" #2() "\n"
			  "read=nobody\nwrite=nobody\nhost=" @host
			  "\naccess=local,remote\n\n"
			]
			[ (ALLPROJ) "  { \"" #2() "\",wise_" #2() "project, wisetdlfield_" #2()
			  ",\"" #6() "\", \"" #3() "\", \"" #8() "\", " #10() ", \"" #12() "\" },\n" ]
			[ (DBDROP)
			  IF (msql_current(#2()) == "1") THEN {
				  "drop table " #2() " \\g\n"
				  "drop table " #2() "data \\g\n"
				  "drop table " #2() "log \\g\n"
			  } ]
			[ (DBDATA)
			  IF (msql_current(#2()) == "1") THEN {
				  "create table " #2() "data (\n"
				  "  propname char(20),\n"
				  "  charval char(20),\n"
				  "  intval int\n) \\g\n\n"
				  "insert into " #2() "data "
				  " (propname, charval, intval) values (\n"
				  "  'MAXISSUENO',\n  '',\n  0\n) \\g\n\n"
			  } ]
			[ (DBLOG)
			  IF (msql_current(#2()) == "1") THEN {
				  "create table " #2() "log (\n"
				  "  clock char(20),\n"
				  "  person char(20),\n"
				  "  issueno int,\n"
				  "  project char(32),\n"
				  &13(DBDECL)
				  ") \\g\n\n"
				  "create index idxissue ON " #2() "log (\n"
				  "  issueno\n"
				  ") \\g\n\n"
			  } ]
			[ (DBDECL)
			  IF (msql_current(#2()) == "1") THEN {
				  "create table " #2() " (\n"
				  "  issueno int,\n"
				  "  project char(32),\n"
				  #13
				  ") \\g\n\n"
				  "create index idxissue ON " #2() " (\n"
				  "  issueno\n"
				  ") \\g\n\n"
			  } ]
			[ (SCRIPT)
			  "echo \"Do you wish to " action() " the " #2() " database on " #8()
				"? [y/n/q]\"\nswitch ($<)\ncase 'y':\n  "
				  "setenv MSQL_TCP_PORT " #10() "\n  "
				  msql_binary() " -h " #8() " " command() " " #2() " < " #2() "-init.msql\n"
				"\n  breaksw\ncase 'q':\n  exit\nendsw\n" ]
			[ (NAMES)
			  "../bin/genconfig -c " #2() " < site.wpl > " #2() "-init.msql\n"
			]
			[ (DUMP)
			  "echo \"drop table " #2() "data\\g\" > " #2() "-$TODAY.msql\n"
			  "echo \"drop table " #2() "log\\g\" >> " #2() "-$TODAY.msql\n"
			  "echo \"drop table " #2() "\\g\" >> " #2() "-$TODAY.msql\n"
			  msql_binary() " " #2() " >> " #2() "-$TODAY.msql\n" ]
		;

fields		:	field
			[ (DBDECL) #1 "\n" ]
		|	field fields
			[ (DBDECL) #1 ",\n" #2 ]
		;

field		:	id fieldtype
			[ (PROJECT) "  { \"" #1() "\"," #2 " },\n" ]
			[ (DBDECL) !fname @fname=#1() #2 ]
		;

fieldtype	:	TEXT num
			[ (PROJECT) "WISE_TEXT," #2() ]
			[ (DBDECL) "  " @fname " char(" #2() ")" ]
		|	TEXTAREA num
			[ (PROJECT) "WISE_TAREA," #2() ]
			[ (DBDECL) "  " @fname " char(" #2() ")" ]
		|	DATE
			[ (PROJECT) "WISE_DATE,20" ]
			[ (DBDECL) "  " @fname " char(20)" ]
		|	SELECT
			[ (PROJECT) "WISE_SELECT,50" ]
			[ (DBDECL) "  " @fname " char(50)" ]
		|	RADIO
			[ (PROJECT) "WISE_RADIO,50" ]
			[ (DBDECL) "  " @fname " char(50)" ]
		|	CHECKBOX
			[ (PROJECT) "WISE_CBOX,50" ]
			[ (DBDECL) "  " @fname " char(50)" ]
		|	HTML num
			[ (PROJECT) "WISE_HTML," #2() ]
			[ (DBDECL)
			  "  " @fname " char(" #2() "),\n"
			  "  " @fname "html char(50)" ]
		|	LOG
			[ (PROJECT) "WISE_LOG,20" ]
			[ (DBDECL) "  " @fname " int" ]
		|	INT
			[ (PROJECT) "WISE_INT,20"  ]
			[ (DBDECL) "  " @fname " int" ]
		|	PASSWD num
			[ (PROJECT) "WISE_PASSWD," #2() ]
			[ (DBDECL) "  " @fname " char(" #2() ")" ]
		;

tdl		:	TDL id '{' tdlfields '}'
			[ (PRJTDL)
			  "WiseTDLField wisetdlfield_" #2() "[] = {\n"
			  #4
			  "  { NULL, NULL }\n"
			  "};\n\n"
			]
		;

tdlfields	:	/* empty */
		|	tdlfield tdlfields
		;

tdlfield	:	id num
			[ (PRJTDL) "  { \"" #1() "\"," #2() " },\n" ]
		;


role            :       ROLE  id ISA id ON PROJECT id ';'
                        [ (ROLE) "  { \"" #2() "\", \"" #4() "\", \"" #7() "\" },\n" ]
			[ (USERS) "echo -n \"Do you want to create a password entry for " #2()
				"? [y/n]\"\nswitch ($<)\ncase 'y':\n  "
				msql_binary() " " create_passwd() " "
				#2() "\n  breaksw\nendsw\n"
				"echo \"require user " #2() "\""
				" >> ../.htaccess\n" ]
		;


tdlview		:	TDLVIEW id SEES ALL ON PROJECT id ';'
			[ (TDLVIEW) "  { \"" #7() "\",\"" #2() "\",\n" "   \"\" },\n" ]
		|	TDLVIEW id SEES when ON PROJECT id ';'
			[ (TDLVIEW) "  { \"" #7() "\",\"" #2() "\",\n" "   " &4(WHEN) " },\n" ]
		;


viewaccess	: 	VIEWACCESS id HAS access ON FIELD id when ON PROJECT id init ';'
			[ (VIEWACCESS)
			  " {\"" #11() "\",\"" #2() "\",\"" #7() "\"," &4(ACCESS) ",\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	VIEWACCESS ALL HAVE access ON FIELD id when ON PROJECT id init ';'
			[ (VIEWACCESS)
			  " {\"" #11() "\", NULL,\"" #7() "\"," &4(ACCESS) ",\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	VIEWACCESS FIELD id IS HIDDEN TO id when ON PROJECT id init ';'
			[ (VIEWACCESS)
			  " {\"" #11() "\",\"" #7() "\",\"" #3() "\", WISE_H ,\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	VIEWACCESS FIELD id IS HIDDEN TO ALL when ON PROJECT id init ';'
			[ (VIEWACCESS)
			  " {\"" #11() "\", NULL,\"" #3() "\", WISE_H ,\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		;


newaccess	:	NEWACCESS id HAS access ON FIELD id when ON PROJECT id init ';'
			[ (NEWACCESS)
			  " {\"" #11() "\",\"" #2() "\",\"" #7() "\"," &4(ACCESS) ",\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	NEWACCESS ALL HAVE access ON FIELD id when ON PROJECT id init ';'
			[ (NEWACCESS)
			 " {\"" #11() "\", NULL,\"" #7() "\"," &4(ACCESS) ",\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	NEWACCESS FIELD id IS HIDDEN TO id when ON PROJECT id init ';'
			[ (NEWACCESS)
			  " {\"" #11() "\",\"" #7() "\",\"" #3() "\", WISE_H ,\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		|	NEWACCESS FIELD id IS HIDDEN TO ALL when ON PROJECT id init ';'
			[ (NEWACCESS)
			  " {\"" #11() "\", NULL,\"" #3() "\", WISE_H ,\n"
			   &8(WHEN) "," &12(INIT) "},\n" 
			]
		;


tdlaction	:	TDLACTION id CAN CALL action label when ON PROJECT id ';'
			[ (TDLACTION) 
			"   { \"" #10() "\",\"" #2() "\",\n" 
			"\t" &7(WHEN) ",\n  \"" &6(LBL) "\",\n"  "\t" &5(ACTION) "},\n" ]
		|	TDLACTION ALL CAN CALL action label when ON PROJECT id ';'
			[ (TDLACTION) 
			"   { \"" #10() "\", NULL,\n"
			"\t" &7(WHEN) ",\n  \"" &6(LBL) "\",\n" "\t" &5(ACTION) "}, \n" ]
		;


viewaction	:	VIEWACTION id CAN CALL action label when ON PROJECT id ';'
			[ (VIEWACTION) 
			"   { \"" #10() "\",\"" #2() "\",\n"
			"\t" &7(WHEN) ",\n  \"" &6(LBL) "\",\n"  "\t" &5(ACTION) "},\n" ]
		|	VIEWACTION ALL CAN CALL action label when ON PROJECT id ';'
			[ (VIEWACTION) 
			"   { \"" #10() "\", NULL,\n"
			"\t" &7(WHEN) ",\n  \"" &6(LBL) "\",\n" "\t" &5(ACTION) "}, \n" ]
		;

label		:	LABEL STRING
			[ (LBL) #2() ]
		;

access		:	READONLY ACCESS
			[ (ACCESS) "WISE_R" ] 
		|	READWRITE ACCESS
			[ (ACCESS) "WISE_RW"]
		|	WRITEONLY ACCESS
			[ (ACCESS) "WISE_W" ]
		;

init		:	/* empty */
			[ (INIT) "NULL" ] 
		|	'=' component
			[ (INIT) "\"" &2(COMP) "\"" ]
		;

idlist		:	id
			[ (LIST) #1()]
		| 	id idlist 
			[ (LIST) #1() " " &2(LIST) ]
 		;

component       :       idlist
                        [ (COMP) &1(LIST) ] 
                |       idlist ',' component
                        [ (COMP) &1(LIST) "|"  &3(COMP) ]
		;

	 
when		:	/* empty */
			[ (WHEN) "NULL" ]
		|	WHEN query
			[ (WHEN) "\"" &2(QUERY) "\""]
		;

query		:	qid
			[ (QUERY) &1(QUERY) ]
		| 	qid AND query
			[ (QUERY) &1(QUERY) " AND " &3(QUERY) ]
		| 	qid OR 	query
			[ (QUERY) &1(QUERY) " OR " &3(QUERY) ]
		;

qid		:	id
		| 	STRING	
		|	column '=' value
			[ (QUERY) #1() " = \'" #3() "\'" ] 
		|	column operator value
			[ (QUERY) #1() " " #2() " \'" #3() "\'" ] 
		;


column 		: 	id
		;

operator	: 	OPERATOR
		| 	LIKE
		;

value 		: 	id 
		| 	num
		;

id		:	ID
		;

num		:	NUM
		;

action		:	NEWISSUE
			[ (ACTION) "\tWISE_NEW " ] 
		|	DELETEISSUE	
			[ (ACTION) "\tWISE_VIEW_DELETE " ] 
		|	SUBMITISSUE	
			[ (ACTION) "\tWISE_VIEW_UPDATE " ] 
		;

%%



#include "lex.yy.c"
